@@ -8,6 +8,9 @@
 
 #include <glib.h>
 
+#include "xx-cutouts-v1-client-protocol.h"
+#include "xdg-shell-client-protocol.h"
+
 #include <wayland-client.h>
 
 G_BEGIN_DECLS
@@ -23,17 +26,18 @@ typedef struct _PtkButtonEvent {
 
 
 typedef struct _PtkDisplay {
-  struct wl_display       *display;
-  struct wl_registry      *registry;
-  struct wl_compositor    *compositor;
-  struct wl_subcompositor *subcompositor;
-  struct wl_seat          *seat;
-  struct wl_shm           *shm;
-  struct wl_pointer       *pointer;
-  struct wl_keyboard      *keyboard;
-  struct wl_touch         *touch;
-  struct wl_output        *wl_output;
-  struct xdg_wm_base      *xdg_wm_base;
+  struct wl_display            *display;
+  struct wl_registry           *registry;
+  struct wl_compositor         *compositor;
+  struct wl_subcompositor      *subcompositor;
+  struct wl_seat               *seat;
+  struct wl_shm *shm;
+  struct wl_pointer            *pointer;
+  struct wl_keyboard           *keyboard;
+  struct wl_touch              *touch;
+  struct wl_output             *wl_output;
+  struct xdg_wm_base           *xdg_wm_base;
+  struct xx_cutouts_manager_v1 *xx_cutouts_manager_v1;
 } PtkDisplay;
 
 PtkDisplay *            ptk_display_init                        (void);
