@@ -114,6 +114,8 @@ typedef struct _PhocDesktopPrivate {
 
   PhocWorkspaceManager  *workspace_manager;
   PhocWorkspace         *active_workspace;
+
+  PhocXxCutoutsManager  *xx_cutouts_manager;
 } PhocDesktopPrivate;
 
 G_DEFINE_TYPE_WITH_PRIVATE (PhocDesktop, phoc_desktop, G_TYPE_OBJECT);
@@ -700,6 +702,7 @@ phoc_desktop_constructed (GObject *object)
   wl_signal_add (&self->xdg_decoration_manager->events.new_toplevel_decoration,
                  &self->xdg_toplevel_decoration);
 
+  priv->xx_cutouts_manager = phoc_xx_cutouts_manager_new ();
   self->xdg_toplevel_decoration.notify = phoc_handle_xdg_toplevel_decoration;
   wlr_viewporter_create (wl_display);
   wlr_single_pixel_buffer_manager_v1_create (wl_display);
@@ -784,6 +787,7 @@ phoc_desktop_finalize (GObject *object)
   g_clear_object (&priv->phosh);
   g_clear_pointer (&priv->gtk_shell, phoc_gtk_shell_destroy);
   g_clear_object (&priv->layer_shell_effects);
+  g_clear_object (&priv->xx_cutouts_manager);
   g_clear_pointer (&self->layout, wlr_output_layout_destroy);
 
   g_clear_object (&priv->outputs_states);
@@ -1277,13 +1281,13 @@ phoc_desktop_is_privileged_protocol (PhocDesktop *self, const struct wl_global *
   priv = phoc_desktop_get_instance_private (self);
 
   is_priv = (
-    global == phoc_phosh_private_get_global (priv->phosh) ||
     global == phoc_layer_shell_effects_get_global (priv->layer_shell_effects) ||
+    global == phoc_phosh_private_get_global (priv->phosh) ||
     global == priv->data_control_manager_v1->global ||
-    global == self->ext_foreign_toplevel_list_v1->global ||
+    global == priv->ext_image_copy_capture_manager_v1->global ||
     global == priv->screencopy_manager_v1->global ||
     global == self->export_dmabuf_manager_v1->global ||
-    global == priv->ext_image_copy_capture_manager_v1->global ||
+    global == self->ext_foreign_toplevel_list_v1->global ||
     global == self->foreign_toplevel_manager_v1->global ||
     global == self->gamma_control_manager_v1->global ||
     global == self->input_method->global ||
@@ -1665,3 +1669,21 @@ phoc_desktop_get_active_workspace (PhocDesktop *self)
 
   return priv->active_workspace;
 }
+
+/**
+ * phoc_desktop_get_xx_cutouts_manager:
+ * @self: the desktop
+ *
+ * Get the cutouts manager
+ *
+ * Returns:(transfer none): The cutouts manager
+ */
+PhocXxCutoutsManager *
+phoc_desktop_get_xx_cutouts_manager (PhocDesktop *self)
+{
+  PhocDesktopPrivate *priv = phoc_desktop_get_instance_private (self);
+
+  g_assert (PHOC_IS_DESKTOP (self));
+
+  return priv->xx_cutouts_manager;
+}
