@@ -28,29 +28,34 @@
 
 typedef void (*PhocKeyHandlerFunc) (PhocSeat *seat, GVariant *param);
 
+G_DEFINE_BOXED_TYPE (PhocKeybindingsContext,
+                     phoc_keybindings_context,
+                     phoc_keybindings_context_copy,
+                     phoc_keybindings_context_free)
+
 /**
  * PhocKeybinding:
  *
  * A keybinding represents a handler with params that will be
  * invoked on the given keybinding combinations.
  */
-typedef struct
-{
-  char               *name;
-  PhocKeyHandlerFunc  func;
-  GVariant           *param;
+typedef struct {
+  char     *name;
+  PhocKeyHandlerFunc func;
+  GVariant *param;
 
-  GSList             *combos;
+  GSList   *combos;
 } PhocKeybinding;
 
 
-typedef struct _PhocKeybindings
-{
-  GObject parent;
+typedef struct _PhocKeybindings {
+  GObject    parent;
 
-  GSList *bindings;
+  GSList    *bindings;
   GSettings *settings;
   GSettings *mutter_settings;
+
+  PhocKeybindingsContext *context;
 } PhocKeybindings;
 
 G_DEFINE_TYPE (PhocKeybindings, phoc_keybindings, G_TYPE_OBJECT);
@@ -153,6 +158,34 @@ handle_cycle_windows_backwards (PhocSeat *seat, GVariant *param)
 }
 
 
+static void
+handle_cycle_workspace_windows (PhocSeat *seat, GVariant *param)
+{
+  PhocView *view;
+  PhocDesktop *desktop = phoc_server_get_desktop (phoc_server_get_default ());
+  PhocWorkspaceManager *manager = phoc_desktop_get_workspace_manager (desktop);
+  PhocWorkspace *workspace = phoc_workspace_manager_get_active (manager);
+
+  view = phoc_workspace_cycle (workspace, TRUE);
+  if (view)
+    phoc_seat_set_focus_view (seat, view);
+}
+
+
+static void
+handle_cycle_workspace_windows_backwards (PhocSeat *seat, GVariant *param)
+{
+  PhocView *view;
+  PhocDesktop *desktop = phoc_server_get_desktop (phoc_server_get_default ());
+  PhocWorkspaceManager *manager = phoc_desktop_get_workspace_manager (desktop);
+  PhocWorkspace *workspace = phoc_workspace_manager_get_active (manager);
+
+  view = phoc_workspace_cycle (workspace, FALSE);
+  if (view)
+    phoc_seat_set_focus_view (seat, view);
+}
+
+
 static void
 handle_close (PhocSeat *seat, GVariant *param)
 {
@@ -435,13 +468,13 @@ is_keycode (const char *string)
 }
 
 /**
- * phoc_parse_accelerator: (skip)
+ * phoc_keybindings_parse_accelerator: (skip)
  *
  * Parse strings representing keybindings into modifier
  * and symbols.
  */
 PhocKeyCombo *
-phoc_parse_accelerator (const char *accelerator)
+phoc_keybindings_parse_accelerator (const char *accelerator, PhocKeybindingsContext *context)
 {
   PhocKeyCombo *combo;
   xkb_keysym_t keyval;
@@ -515,12 +548,16 @@ phoc_parse_accelerator (const char *accelerator)
       }
     } else {
       if (len >= 4 && is_keycode (accelerator)) {
-        //keycode = strtoul (accelerator, NULL, 16);
+        // keycode = strtoul (accelerator, NULL, 16);
         g_warning ("Unhandled keycode accelerator'");
         goto out;
       } else if (strcmp (accelerator, "Above_Tab") == 0) {
-        g_warning ("Unhandled key 'Above_Tab'");
-        return FALSE;
+        if (context && context->above_tab_keysym != XKB_KEY_NoSymbol) {
+          keyval = context->above_tab_keysym;
+        } else {
+          g_warning ("Unhandled key 'Above_Tab'");
+          return FALSE;
+        }
       } else {
         keyval = xkb_keysym_from_name (accelerator, XKB_KEYSYM_CASE_INSENSITIVE);
         if (keyval == XKB_KEY_NoSymbol) {
@@ -545,6 +582,20 @@ phoc_parse_accelerator (const char *accelerator)
 }
 
 
+static PhocKeybinding *
+phoc_keybinding_new (const char *name, PhocKeyHandlerFunc func, GVariant *param)
+{
+  PhocKeybinding *binding = g_new0 (PhocKeybinding, 1);
+
+  binding->name = g_strdup (name);
+  binding->func = func;
+  if (param)
+    binding->param = g_variant_ref_sink (param);
+
+  return binding;
+}
+
+
 static void
 phoc_keybinding_free (PhocKeybinding *keybinding)
 {
@@ -583,25 +634,18 @@ keybinding_by_key_combo (const PhocKeybinding *keybinding, const PhocKeyCombo *c
 
 
 static void
-on_keybinding_setting_changed (PhocKeybindings *self,
-                               const char      *key,
-                               GSettings       *settings)
+phoc_keybindings_update_accelerators (PhocKeybindings    *self,
+                                      const char         *name,
+                                      const char * const *accelerators)
 {
-  g_auto(GStrv) accelerators = NULL;
   PhocKeybinding *keybinding;
   GSList *elem;
-  int i;
-
-  g_return_if_fail (PHOC_IS_KEYBINDINGS (self));
-  g_return_if_fail (G_IS_SETTINGS (settings));
-
-  accelerators = g_settings_get_strv (settings, key);
 
   elem = g_slist_find_custom (self->bindings,
-                              key,
+                              name,
                               (GCompareFunc)keybinding_by_name);
   if (!elem) {
-    g_warning ("Changed keybinding %s not known", key);
+    g_warning ("Changed keybinding %s not known", name);
     return;
   }
 
@@ -610,23 +654,38 @@ on_keybinding_setting_changed (PhocKeybindings *self,
   g_slist_free_full (keybinding->combos, g_free);
   keybinding->combos = NULL;
 
-  for (i = 0; accelerators && accelerators[i]; i++) {
+  for (int i = 0; accelerators && accelerators[i]; i++) {
     PhocKeyCombo *combo;
 
-    g_debug ("New keybinding %s for %s", key, accelerators[i]);
-    combo = phoc_parse_accelerator (accelerators[i]);
+    g_debug ("New keybinding %s for %s", name, accelerators[i]);
+    combo = phoc_keybindings_parse_accelerator (accelerators[i], self->context);
     if (combo)
       keybinding->combos = g_slist_append (keybinding->combos, combo);
   }
 }
 
 
+static void
+on_keybinding_setting_changed (PhocKeybindings *self,
+                               const char      *name,
+                               GSettings       *settings)
+{
+  g_auto (GStrv) accelerators = NULL;
+
+  g_return_if_fail (PHOC_IS_KEYBINDINGS (self));
+  g_return_if_fail (G_IS_SETTINGS (settings));
+
+  accelerators = g_settings_get_strv (settings, name);
+  phoc_keybindings_update_accelerators (self, name, (const char * const *) accelerators);
+}
+
+
 static gboolean
-phoc_add_keybinding (PhocKeybindings    *self,
-                     GSettings          *settings,
-                     const char         *name,
-                     PhocKeyHandlerFunc  func,
-                     GVariant           *param)
+phoc_add_keybinding (PhocKeybindings   *self,
+                     GSettings         *settings,
+                     const char        *name,
+                     PhocKeyHandlerFunc func,
+                     GVariant          *param)
 {
   g_autofree char *signal_name = NULL;
   PhocKeybinding *binding;
@@ -636,11 +695,7 @@ phoc_add_keybinding (PhocKeybindings    *self,
     return FALSE;
   }
 
-  binding = g_new0 (PhocKeybinding, 1);
-  binding->name = g_strdup (name);
-  binding->func = func;
-  if (param)
-    binding->param = g_variant_ref_sink (param);
+  binding = phoc_keybinding_new (name, func, param);
 
   signal_name = g_strdup_printf ("changed::%s", name);
   g_signal_connect_swapped (settings, signal_name,
@@ -670,6 +725,7 @@ phoc_keybindings_finalize (GObject *object)
 {
   PhocKeybindings *self = PHOC_KEYBINDINGS (object);
 
+  g_clear_pointer (&self->context, phoc_keybindings_context_free);
   g_clear_object (&self->settings);
   g_clear_object (&self->mutter_settings);
 
@@ -677,14 +733,12 @@ phoc_keybindings_finalize (GObject *object)
 }
 
 
-static void
-phoc_keybindings_constructed (GObject *object)
+void
+phoc_keybindings_load_settings (PhocKeybindings *self)
 {
-  PhocKeybindings *self = PHOC_KEYBINDINGS (object);
+  g_slist_free_full (self->bindings, (GDestroyNotify)phoc_keybinding_free);
+  self->bindings = NULL;
 
-  G_OBJECT_CLASS (phoc_keybindings_parent_class)->constructed (object);
-
-  self->settings = g_settings_new (KEYBINDINGS_SCHEMA_ID);
   phoc_add_keybinding (self, self->settings, "always-on-top", handle_always_on_top, NULL);
   phoc_add_keybinding (self, self->settings, "close", handle_close, NULL);
   phoc_add_keybinding (self, self->settings, "cycle-windows", handle_cycle_windows, NULL);
@@ -724,6 +778,12 @@ phoc_keybindings_constructed (GObject *object)
   phoc_add_keybinding (self, self->settings,
                        "switch-applications-backward", handle_cycle_windows_backwards,
                        NULL);
+  phoc_add_keybinding (self, self->settings,
+                       "switch-windows", handle_cycle_workspace_windows,
+                       NULL);
+  phoc_add_keybinding (self, self->settings,
+                       "switch-windows-backward", handle_cycle_workspace_windows_backwards,
+                       NULL);
   phoc_add_keybinding (self, self->settings,"unmaximize", handle_unmaximize, NULL);
   phoc_add_keybinding (self, self->settings,
                        "switch-input-source", handle_switch_input_source,
@@ -771,7 +831,6 @@ phoc_keybindings_constructed (GObject *object)
                        "move-to-workspace-right", handle_move_to_workspace_relative,
                        g_variant_new_int32 (+1));
 
-  self->mutter_settings = g_settings_new (MUTTER_KEYBINDINGS_SCHEMA_ID);
   phoc_add_keybinding (self, self->mutter_settings,
                        "toggle-tiled-left", handle_tile,
                        g_variant_new_int32 (PHOC_VIEW_TILE_LEFT));
@@ -786,7 +845,6 @@ phoc_keybindings_class_init (PhocKeybindingsClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-  object_class->constructed = phoc_keybindings_constructed;
   object_class->dispose = phoc_keybindings_dispose;
   object_class->finalize = phoc_keybindings_finalize;
 }
@@ -796,6 +854,8 @@ static void
 phoc_keybindings_init (PhocKeybindings *self)
 {
   self->bindings = NULL;
+  self->settings = g_settings_new (KEYBINDINGS_SCHEMA_ID);
+  self->mutter_settings = g_settings_new (MUTTER_KEYBINDINGS_SCHEMA_ID);
 }
 
 
@@ -846,3 +906,44 @@ phoc_keybindings_handle_pressed (PhocKeybindings *self,
   (*keybinding->func) (seat, keybinding->param);
   return TRUE;
 }
+
+
+void
+phoc_keybindings_set_context (PhocKeybindings *self, PhocKeybindingsContext *context)
+{
+  g_assert (PHOC_IS_KEYBINDINGS (self));
+
+  g_clear_pointer (&self->context, phoc_keybindings_context_free);
+
+  if (context == NULL)
+    return;
+
+  self->context = phoc_keybindings_context_copy (context);
+}
+
+
+PhocKeybindingsContext *
+phoc_keybindings_context_new (void)
+{
+  return g_new0 (PhocKeybindingsContext, 1);
+}
+
+
+void
+phoc_keybindings_context_free (PhocKeybindingsContext *context)
+{
+  g_free (context);
+}
+
+
+PhocKeybindingsContext *
+phoc_keybindings_context_copy (PhocKeybindingsContext *context)
+{
+  g_autoptr (PhocKeybindingsContext) copy = phoc_keybindings_context_new ();
+
+  g_assert (context);
+
+  copy->above_tab_keysym = context->above_tab_keysym;
+
+  return g_steal_pointer (&copy);
+}
