@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2021 Purism SPC
- *               2025 The Phosh Developers
+ *               2025 Phosh.mobi e.V.
  *
  * SPDX-License-Identifier: GPL-3.0-or-later
  *
@@ -71,6 +71,7 @@ struct _PhocKeyboard {
   GSettings         *keyboard_settings;
   struct xkb_keymap *keymap;
   GnomeXkbInfo      *xkbinfo;
+  PhocKeybindings   *keybindings;
 
   gboolean           wakeup_key_default;
   GHashTable        *wakeup_keys;
@@ -262,7 +263,6 @@ keyboard_execute_binding (PhocKeyboard              *self,
                           size_t                     keysyms_len,
                           enum wl_keyboard_key_state state)
 {
-  PhocConfig *config = phoc_server_get_config (phoc_server_get_default ());
   PhocSeat *seat = phoc_input_device_get_seat (PHOC_INPUT_DEVICE (self));
 
   if (state != WL_KEYBOARD_KEY_STATE_PRESSED)
@@ -275,7 +275,7 @@ keyboard_execute_binding (PhocKeyboard              *self,
   }
 
   size_t n = pressed_keysyms_length (pressed_keysyms);
-  if (phoc_keybindings_handle_pressed (config->keybindings, modifiers, pressed_keysyms, n, seat))
+  if (phoc_keybindings_handle_pressed (self->keybindings, modifiers, pressed_keysyms, n, seat))
     return true;
 
   return false;
@@ -535,11 +535,36 @@ set_xkb_keymap (PhocKeyboard *self, const char *layout, const char *variant, con
 }
 
 
+#define BUFLEN 256
+static xkb_keysym_t
+get_above_tab_keysym (PhocKeyboard *keyboard)
+{
+  guint keycode = 0x29 /* KEY_GRAVE */ + 8;
+  const xkb_keysym_t *keysyms;
+  size_t keysyms_len;
+  uint32_t modifiers;
+  char namebuf[BUFLEN];
+
+  g_assert (PHOC_IS_KEYBOARD (keyboard));
+
+  keysyms_len = keyboard_keysyms_raw (keyboard, keycode, &keysyms, &modifiers);
+  if (keysyms_len < 1) {
+    g_warning ("No Above_Tab (keycode 0x%x) keysym entries in layout", keycode);
+    return XKB_KEY_NoSymbol;
+  }
+
+  xkb_keysym_get_name (keysyms[0], namebuf, BUFLEN);
+  g_debug ("Above_Tab set to keysym 0x%x (%s)", keysyms[0], namebuf);
+  return keysyms[0];
+}
+
+
 static void
 on_input_setting_changed (PhocKeyboard *self,
                           const char   *key,
                           GSettings    *settings)
 {
+  g_autoptr (PhocKeybindingsContext) context = phoc_keybindings_context_new ();
   g_auto (GStrv) xkb_options = NULL;
   g_autoptr (GVariant) sources = NULL;
   GVariantIter iter;
@@ -588,6 +613,10 @@ on_input_setting_changed (PhocKeyboard *self,
   g_debug ("Switching to layout %s %s", layout, variant);
 
   set_xkb_keymap (self, layout, variant, xkb_options_string);
+
+  context->above_tab_keysym = get_above_tab_keysym (self);
+  phoc_keybindings_set_context (self->keybindings, context);
+  phoc_keybindings_load_settings (self->keybindings);
 }
 
 
@@ -661,6 +690,7 @@ phoc_keyboard_dispose (GObject *object)
   g_clear_object (&self->input_settings);
   g_clear_object (&self->keyboard_settings);
   g_clear_object (&self->xkbinfo);
+  g_clear_object (&self->keybindings);
 
   G_OBJECT_CLASS (phoc_keyboard_parent_class)->dispose (object);
 }
@@ -796,16 +826,16 @@ phoc_keyboard_constructed (GObject *object)
   self->xkbinfo = gnome_xkb_info_new ();
 
   g_object_connect (self->input_settings,
-    "swapped-signal::changed::sources", G_CALLBACK (on_input_setting_changed), self,
-    "swapped-signal::changed::xkb-options", G_CALLBACK (on_input_setting_changed), self,
-    NULL);
+                    "swapped-signal::changed::sources", on_input_setting_changed, self,
+                    "swapped-signal::changed::xkb-options", on_input_setting_changed, self,
+                    NULL);
   on_input_setting_changed (self, NULL, self->input_settings);
 
   g_object_connect (self->keyboard_settings,
-    "swapped-signal::changed::repeat", G_CALLBACK (on_keyboard_setting_changed), self,
-    "swapped-signal::changed::repeat-interval", G_CALLBACK (on_keyboard_setting_changed), self,
-    "swapped-signal::changed::delay", G_CALLBACK (on_keyboard_setting_changed), self,
-    NULL);
+                    "swapped-signal::changed::repeat", on_keyboard_setting_changed, self,
+                    "swapped-signal::changed::repeat-interval", on_keyboard_setting_changed, self,
+                    "swapped-signal::changed::delay", on_keyboard_setting_changed, self,
+                    NULL);
   on_keyboard_setting_changed (self, NULL, self->keyboard_settings);
 }
 
@@ -837,6 +867,7 @@ phoc_keyboard_class_init (PhocKeyboardClass *klass)
 static void
 phoc_keyboard_init (PhocKeyboard *self)
 {
+  self->keybindings = phoc_keybindings_new ();
 }
 
 
