@@ -83,24 +83,26 @@ static GDebugKey debug_keys[] =
 {
   { .key = "auto-maximize",
     .value = PHOC_SERVER_DEBUG_FLAG_AUTO_MAXIMIZE,},
+  { .key = "cutouts",
+    .value = PHOC_SERVER_DEBUG_FLAG_CUTOUTS,},
   { .key = "damage-tracking",
     .value = PHOC_SERVER_DEBUG_FLAG_DAMAGE_TRACKING,},
   { .key = "damage-whole",
     .value = PHOC_SERVER_DEBUG_FLAG_DAMAGE_WHOLE,},
-  { .key = "no-quit",
-    .value = PHOC_SERVER_DEBUG_FLAG_NO_QUIT,},
-  { .key = "touch-points",
-    .value = PHOC_SERVER_DEBUG_FLAG_TOUCH_POINTS,},
-  { .key = "layer-shell",
-    .value = PHOC_SERVER_DEBUG_FLAG_LAYER_SHELL,},
-  { .key = "cutouts",
-    .value = PHOC_SERVER_DEBUG_FLAG_CUTOUTS,},
   { .key = "disable-animations",
     .value = PHOC_SERVER_DEBUG_FLAG_DISABLE_ANIMATIONS,},
+  { .key = "fake-builtin",
+    .value = PHOC_SERVER_DEBUG_FLAG_FAKE_BUILTIN,},
   { .key = "force-shell-reveal",
     .value = PHOC_SERVER_DEBUG_FLAG_FORCE_SHELL_REVEAL,},
   { .key = "ignore-state",
     .value = PHOC_SERVER_DEBUG_FLAG_IGNORE_STATES,},
+  { .key = "layer-shell",
+    .value = PHOC_SERVER_DEBUG_FLAG_LAYER_SHELL,},
+  { .key = "no-quit",
+    .value = PHOC_SERVER_DEBUG_FLAG_NO_QUIT,},
+  { .key = "touch-points",
+    .value = PHOC_SERVER_DEBUG_FLAG_TOUCH_POINTS,},
 };
 
 
