@@ -187,6 +187,7 @@ const char *phoc_output_get_identifier       (PhocOutput *self);
 void       phoc_output_transform_damage      (PhocOutput *self, pixman_region32_t *damage);
 void       phoc_output_transform_box         (PhocOutput *self, struct wlr_box *box);
 GSList    *phoc_output_get_debug_damage      (PhocOutput *self);
+void       phoc_output_set_fullscreen_view   (PhocOutput *self, PhocView *view);
 
 enum wlr_scale_filter_mode
            phoc_output_get_texture_filter_mode (PhocOutput *self);
@@ -194,5 +195,12 @@ enum wlr_scale_filter_mode
 void       phoc_output_add_bling (PhocOutput *self, PhocBling *bling);
 void       phoc_output_remove_bling (PhocOutput *self, PhocBling *bling);
 GSList *   phoc_output_get_blings (PhocOutput *self);
+gboolean   phoc_output_get_cutout_boxes   (PhocOutput        *self,
+                                           PhocView          *view,
+                                           pixman_region32_t *overlap);
+GArray *   phoc_output_get_cutout_corners (PhocOutput        *self,
+                                           PhocView          *view);
+
+
 
 G_END_DECLS
