@@ -14,7 +14,7 @@
 #include "debug-control.h"
 #include "render-private.h"
 #include "seat.h"
-#include "server.h"
+#include "server-private.h"
 #include "surface.h"
 #include "utils.h"
 
@@ -381,6 +381,7 @@ phoc_server_initable_init (GInitable    *initable,
     return FALSE;
   }
   wl_display_set_global_filter (self->wl_display, phoc_server_filter_globals, self);
+  wl_display_set_default_max_buffer_size (self->wl_display, 1024 * 1024);
 
   self->backend = wlr_backend_autocreate (wl_display_get_event_loop (self->wl_display),
                                           &self->session);
@@ -889,6 +890,16 @@ phoc_server_get_compatibles (PhocServer *self)
 }
 
 
+void
+phoc_server_set_compatibles (PhocServer *self, const char *const *compatibles)
+{
+  g_assert (PHOC_IS_SERVER (self));
+
+  g_strfreev (self->dt_compatibles);
+  self->dt_compatibles = g_strdupv ((GStrv)compatibles);
+}
+
+
 struct wl_display *
 phoc_server_get_wl_display (PhocServer *self)
 {
@@ -932,6 +943,7 @@ phoc_server_set_linux_dmabuf_surface_feedback (PhocServer *self,
                                                bool        enable)
 {
   g_assert (PHOC_IS_SERVER (self));
+  g_assert (PHOC_IS_VIEW (view));
 
   if (!self->linux_dmabuf_v1 || !view->wlr_surface)
     return;
