@@ -98,7 +98,7 @@ G_DEFINE_TYPE_WITH_CODE (PhocView, phoc_view, G_TYPE_OBJECT,
 
 #define PHOC_VIEW_SELF(p) PHOC_PRIV_CONTAINER(PHOC_VIEW, PhocView, (p))
 
-static bool view_center (PhocView *view, PhocOutput *output);
+static bool view_center (PhocView *self, PhocOutput *output);
 
 /* {{{ PhocChildRoot interface */
 
@@ -204,7 +204,7 @@ phoc_view_child_root_unconstrain_popup (PhocChildRoot *root, struct wlr_box *box
   usable_area.x += output->lx;
   usable_area.y += output->ly;
 
-  /* the output box expressed in the coordinate system of the toplevel parent
+  /* The output box expressed in the coordinate system of the toplevel parent
    * of the popup */
   *box = (struct wlr_box) {
     .x = usable_area.x - self->box.x,
@@ -328,19 +328,20 @@ view_create_foreign_toplevel_handle (PhocView *self)
   g_assert (priv->toplevel_handle);
 
   priv->toplevel_handle_request_maximize.notify = handle_toplevel_handle_request_maximize;
-  wl_signal_add(&priv->toplevel_handle->events.request_maximize,
-                &priv->toplevel_handle_request_maximize);
+  wl_signal_add (&priv->toplevel_handle->events.request_maximize,
+                 &priv->toplevel_handle_request_maximize);
 
   priv->toplevel_handle_request_activate.notify = handle_toplevel_handle_request_activate;
-  wl_signal_add(&priv->toplevel_handle->events.request_activate,
-                &priv->toplevel_handle_request_activate);
+  wl_signal_add (&priv->toplevel_handle->events.request_activate,
+                 &priv->toplevel_handle_request_activate);
 
   priv->toplevel_handle_request_fullscreen.notify = handle_toplevel_handle_request_fullscreen;
-  wl_signal_add(&priv->toplevel_handle->events.request_fullscreen,
-                &priv->toplevel_handle_request_fullscreen);
+  wl_signal_add (&priv->toplevel_handle->events.request_fullscreen,
+                 &priv->toplevel_handle_request_fullscreen);
 
   priv->toplevel_handle_request_close.notify = handle_toplevel_handle_request_close;
-  wl_signal_add(&priv->toplevel_handle->events.request_close, &priv->toplevel_handle_request_close);
+  wl_signal_add (&priv->toplevel_handle->events.request_close,
+                 &priv->toplevel_handle_request_close);
 
   priv->toplevel_handle->data = self;
 
@@ -367,11 +368,10 @@ phoc_view_destroy_toplevel_handle (PhocView *self)
   wl_list_remove (&priv->toplevel_handle_request_activate.link);
   wl_list_remove (&priv->toplevel_handle_request_fullscreen.link);
   wl_list_remove (&priv->toplevel_handle_request_close.link);
-  wlr_foreign_toplevel_handle_v1_destroy (priv->toplevel_handle);
-  priv->toplevel_handle = NULL;
 
-  wlr_ext_foreign_toplevel_handle_v1_destroy (priv->ext_foreign_toplevel_v1_handle);
-  priv->ext_foreign_toplevel_v1_handle = NULL;
+  g_clear_pointer (&priv->toplevel_handle, wlr_foreign_toplevel_handle_v1_destroy);
+  g_clear_pointer (&priv->ext_foreign_toplevel_v1_handle,
+                   wlr_ext_foreign_toplevel_handle_v1_destroy);
 }
 
 
@@ -449,18 +449,81 @@ phoc_view_get_fullscreen_output (PhocView *self)
   return priv->fullscreen_output;
 }
 
+/**
+ * phoc_view_get_box:
+ * @self: The view
+ * @box: (out): The box
+ *
+ * Get the view's box in layout coordinates (taking any scale-to-fit
+ * scale into account).
+ */
 void
-phoc_view_get_box (PhocView *view, struct wlr_box *box)
+phoc_view_get_box (PhocView *self, struct wlr_box *box)
 {
   PhocViewPrivate *priv;
 
-  g_assert (PHOC_IS_VIEW (view));
-  priv = phoc_view_get_instance_private (view);
+  g_assert (PHOC_IS_VIEW (self));
+  priv = phoc_view_get_instance_private (self);
 
-  box->x = view->box.x;
-  box->y = view->box.y;
-  box->width = view->box.width * priv->scale;
-  box->height = view->box.height * priv->scale;
+  box->x = self->box.x;
+  box->y = self->box.y;
+  box->width = self->box.width * priv->scale;
+  box->height = self->box.height * priv->scale;
+}
+
+/**
+ * phoc_view_get_pending_box:
+ * @self: The view
+ *
+ * Get the view's box in layout coordinates taking any pending move
+ * and resize operations into account. Note that this rounds x,y to
+ * their lower values. If there's currently no update pending we
+ * return the current box for convenience.
+ *
+ * Returns: The pending box
+ */
+PhocBox
+phoc_view_get_pending_box (PhocView *self)
+{
+  PhocBox box;
+
+  g_assert (PHOC_IS_VIEW (self));
+
+  box.x = self->pending_move_resize.x;
+  box.y = self->pending_move_resize.y;
+  box.width = self->pending_move_resize.width;
+  box.height = self->pending_move_resize.height;
+
+  if (box.x == 0.0 && box.y == 0.0 && box.width == 0 && box.height == 0)
+    phoc_view_get_box (self, &box);
+
+  return box;
+}
+
+/**
+ * phoc_view_set_pending_box:
+ * @self: The view
+ * @box: (out): The box
+ *
+ * Set the view's pending box in layout coordinates.
+ */
+void
+phoc_view_set_pending_box (PhocView *self,
+                           bool      update_x,
+                           bool      update_y,
+                           double    x,
+                           double    y,
+                           uint32_t  width,
+                           uint32_t  height)
+{
+  g_assert (PHOC_IS_VIEW (self));
+
+  self->pending_move_resize.update_x = update_x;
+  self->pending_move_resize.update_y = update_y;
+  self->pending_move_resize.x = x;
+  self->pending_move_resize.y = y;
+  self->pending_move_resize.width = width;
+  self->pending_move_resize.height = height;
 }
 
 
@@ -542,7 +605,7 @@ view_save (PhocView *view)
   if (!phoc_view_is_floating (view))
     return;
 
-  /* backup window state */
+  /* Backup window state */
   struct wlr_box geom;
   phoc_view_get_geometry (view, &geom);
   view->saved.x = view->box.x + geom.x * priv->scale;
@@ -910,18 +973,19 @@ phoc_view_restore (PhocView *self)
 void
 phoc_view_set_fullscreen (PhocView *view, bool fullscreen, PhocOutput *output)
 {
-  PhocInput *input = phoc_server_get_input (phoc_server_get_default ());
-  PhocViewPrivate *priv;
+  PhocServer *server = phoc_server_get_default ();
+  PhocInput *input = phoc_server_get_input (server);
+  PhocViewPrivate *priv = phoc_view_get_instance_private (view);
 
   g_assert (PHOC_IS_VIEW (view));
-  priv = phoc_view_get_instance_private (view);
 
   bool was_fullscreen = phoc_view_is_fullscreen (view);
-
   if (was_fullscreen != fullscreen) {
-    /* don't allow unfocused surfaces to make themselves fullscreen */
-    if (fullscreen && phoc_view_is_mapped (view))
-      g_return_if_fail (phoc_input_view_has_focus (input, view));
+    /* Don't allow unfocused surfaces to make themselves fullscreen */
+    if (fullscreen && phoc_view_is_mapped (view) && !phoc_input_view_has_focus (input, view)) {
+      g_warning ("Can't fullscreen view %p without focus", view);
+      return;
+    }
 
     PHOC_VIEW_GET_CLASS (view)->set_fullscreen (view, fullscreen);
 
@@ -933,15 +997,22 @@ phoc_view_set_fullscreen (PhocView *view, bool fullscreen, PhocOutput *output)
   phoc_view_get_geometry (view, &view_geom);
 
   if (fullscreen) {
-    PhocDesktop *desktop = phoc_server_get_desktop (phoc_server_get_default ());
+    PhocDesktop *desktop = phoc_server_get_desktop (server);
 
     if (output == NULL)
       output = phoc_view_get_output (view);
 
-    if (was_fullscreen)
-      priv->fullscreen_output->fullscreen_view = NULL;
+    /* Nothing to do if old fullscreen output is the new one */
+    if (was_fullscreen && output == priv->fullscreen_output)
+      return;
 
-    view_save (view);
+    if (was_fullscreen) {
+      /* If switching fullscreen outputs clear the old one */
+      phoc_output_set_fullscreen_view (priv->fullscreen_output, NULL);
+    } else {
+      /* If not yet fullscreen save the non-fullscreen geometry */
+      view_save (view);
+    }
 
     struct wlr_box output_box;
     wlr_output_layout_get_box (desktop->layout, output->wlr_output, &output_box);
@@ -951,23 +1022,20 @@ phoc_view_set_fullscreen (PhocView *view, bool fullscreen, PhocOutput *output)
                            output_box.width,
                            output_box.height);
 
-    output->fullscreen_view = view;
-    phoc_output_force_shell_reveal (output, false);
     priv->fullscreen_output = output;
-    phoc_output_damage_whole (output);
+    phoc_output_set_fullscreen_view (output, view);
   }
 
   if (was_fullscreen && !fullscreen) {
-    PhocOutput *phoc_output = priv->fullscreen_output;
-    priv->fullscreen_output->fullscreen_view = NULL;
-    priv->fullscreen_output = NULL;
+    PhocOutput *current_output = priv->fullscreen_output;
 
-    phoc_output_damage_whole (phoc_output);
+    priv->fullscreen_output = NULL;
+    phoc_output_set_fullscreen_view (current_output, NULL);
 
     if (priv->state == PHOC_VIEW_STATE_MAXIMIZED) {
-      view_arrange_maximized (view, phoc_output);
+      view_arrange_maximized (view, current_output);
     } else if (priv->state == PHOC_VIEW_STATE_TILED) {
-      view_arrange_tiled (view, phoc_output);
+      view_arrange_tiled (view, current_output);
     } else if (!wlr_box_empty (&view->saved)) {
       phoc_view_move_resize (view,
                              view->saved.x - view_geom.x * priv->scale,
@@ -982,8 +1050,7 @@ phoc_view_set_fullscreen (PhocView *view, bool fullscreen, PhocOutput *output)
     phoc_view_auto_maximize (view);
   }
 
-  phoc_server_set_linux_dmabuf_surface_feedback (phoc_server_get_default (),
-                                                 view, priv->fullscreen_output, fullscreen);
+  phoc_server_set_linux_dmabuf_surface_feedback (server, view, priv->fullscreen_output, fullscreen);
 }
 
 
@@ -1002,7 +1069,7 @@ phoc_view_move_to_next_output (PhocView *view, enum wlr_direction direction)
   if (!output)
     return false;
 
-  /* use current view's x,y as ref_lx, ref_ly */
+  /* Use current view's x,y as ref_lx, ref_ly */
   new_output = wlr_output_layout_adjacent_output (layout, direction, output->wlr_output,
                                                   view->box.x, view->box.y);
   if (!new_output)
@@ -1012,7 +1079,7 @@ phoc_view_move_to_next_output (PhocView *view, enum wlr_direction direction)
   usable_area = output->usable_area;
   l_output = wlr_output_layout_get (desktop->layout, new_output);
 
-  /* update saved position to the new output */
+  /* Update saved position to the new output */
   x = usable_area.x + l_output->x + usable_area.width / 2 - view->saved.width / 2;
   y = usable_area.y + l_output->y + usable_area.height / 2 - view->saved.height / 2;
   g_debug ("moving view's saved position to %f %f", x, y);
@@ -1114,41 +1181,40 @@ phoc_view_tile (PhocView *self, PhocViewTileDirection direction, PhocOutput *out
 
 
 static bool
-view_center (PhocView *view, PhocOutput *output)
+view_center (PhocView *self, PhocOutput *output)
 {
   PhocInput *input = phoc_server_get_input (phoc_server_get_default ());
   PhocDesktop *desktop = phoc_server_get_desktop (phoc_server_get_default ());
+  const struct wlr_output_layout_output *l_output;
   struct wlr_box box, geom;
   PhocViewPrivate *priv;
-
-  g_assert (PHOC_IS_VIEW (view));
-  priv = phoc_view_get_instance_private (view);
-  phoc_view_get_box (view, &box);
-  phoc_view_get_geometry (view, &geom);
-
-  if (!phoc_view_is_floating (view))
-    return false;
-
-  PhocSeat *seat = phoc_input_get_last_active_seat (input);
+  PhocSeat *seat;
   PhocCursor *cursor;
 
-  if (!seat)
+  g_assert (PHOC_IS_VIEW (self));
+  priv = phoc_view_get_instance_private (self);
+  phoc_view_get_box (self, &box);
+  phoc_view_get_geometry (self, &geom);
+
+  if (!phoc_view_is_floating (self))
     return false;
 
+  seat = phoc_input_get_last_active_seat (input);
+  if (!seat)
+    return false;
   cursor = phoc_seat_get_cursor (seat);
 
   if (!output) {
     struct wlr_output *wlr_output = wlr_output_layout_output_at (desktop->layout,
                                                                  cursor->cursor->x,
                                                                  cursor->cursor->y);
-    /* empty layout */
+    /* Empty layout */
     if (!wlr_output)
       return false;
     output = PHOC_OUTPUT (wlr_output->data);
   }
 
-  const struct wlr_output_layout_output *l_output = wlr_output_layout_get (desktop->layout,
-                                                                           output->wlr_output);
+  l_output = wlr_output_layout_get (desktop->layout, output->wlr_output);
   struct wlr_box usable_area = output->usable_area;
 
   double view_x = (double)(usable_area.width - box.width) / 2 +
@@ -1157,18 +1223,18 @@ view_center (PhocView *view, PhocOutput *output)
     usable_area.y + l_output->y - geom.y * priv->scale;
 
   g_debug ("moving view to %f %f", view_x, view_y);
-  phoc_view_move (view, view_x / priv->scale, view_y / priv->scale);
+  phoc_view_move (self, view_x / priv->scale, view_y / priv->scale);
 
   if (!desktop->maximize) {
     // TODO: fitting floating oversized windows requires more work (!228)
     return true;
   }
 
-  if (view->box.width > output->usable_area.width ||
-      view->box.height > output->usable_area.height) {
-    phoc_view_resize (view,
-                      MIN (view->box.width, output->usable_area.width),
-                      MIN (view->box.height, output->usable_area.height));
+  if (self->box.width > output->usable_area.width ||
+      self->box.height > output->usable_area.height) {
+    phoc_view_resize (self,
+                      MIN (self->box.width, output->usable_area.width),
+                      MIN (self->box.height, output->usable_area.height));
   }
 
   return true;
@@ -1202,7 +1268,6 @@ static char *
 munge_app_id (const char *app_id)
 {
   char *id = g_strdup (app_id);
-  gint i;
 
   g_strcanon (id,
               "0123456789"
@@ -1210,7 +1275,7 @@ munge_app_id (const char *app_id)
               "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
               "-",
               '-');
-  for (i = 0; id[i] != '\0'; i++)
+  for (int i = 0; id[i] != '\0'; i++)
     id[i] = g_ascii_tolower (id[i]);
 
   return id;
@@ -1755,7 +1820,7 @@ static void
 phoc_view_set_tiled_default (PhocView *self, bool tiled)
 {
   if (tiled) {
-    /* fallback to the maximized flag on the toplevel so it can remove its drop shadows */
+    /* Fallback to the maximized flag on the toplevel so it can remove its drop shadows */
     PHOC_VIEW_GET_CLASS (self)->set_maximized (self, true);
   }
 }
