@@ -194,8 +194,8 @@ test_phoc_property_easer_props_variant (void)
 }
 
 
-gint
-main (gint argc, char *argv[])
+int
+main (int argc, char *argv[])
 {
   g_test_init (&argc, &argv, NULL);
 
