@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2023 The Phosh Developers
+ * Copyright (C) 2023 Phosh.mobi e.V.
  *
  * SPDX-License-Identifier: GPL-3.0-or-later
  *
@@ -69,8 +69,8 @@ test_phoc_config_modelines (void)
 }
 
 
-gint
-main (gint argc, char *argv[])
+int
+main (int argc, char *argv[])
 {
   g_test_init (&argc, &argv, NULL);
 
